from .colors import CheckColor
from math import pi

class Shape:
    'Base class representing 3D shapes'
    
    def __init__(self,color):
        CheckColor(color)
        self.color = color
        
    def __repr__(self):
        return f'I am a {self.color} shape'

    def volume(self):
        'Computes the volume of this shape.'
        pass

class RectangularPrism(Shape):
    'Class representing a rectangular 3d shape'
    
    def __init__(self,color,length,width,height):
        super().__init__(color)
        self.length = length
        self.width = width
        self.height = height
        
    def __repr__(self):
        return f"{super().__repr__().replace('shape','rectangular prism')}, with side lengths: {self.length},{self.width},{self.height}."
    
    def volume(self):
        return self.length * self.width * self.height
    
    # bonus
    def area(self):
        return self.length * self.width * 2 + self.length * self.height * 2 + self.width * self.height * 2
    
class Cylinder(Shape):
    'Class representing a cylinder shape'
    
    def __init__(self,color,radius,height):
        super().__init__(color)
        self.radius = radius
        self.height = height
    
    def __repr__(self):
        return f"{super().__repr__().replace('shape','cylinder')}, with radius {self.radius}, and height {self.height}"

    def volume(self):
        return pi * self.radius**2 * self.height
    
    #bonus
    def area(self):
        return pi * self.radius * 2 * self.height + pi * self.radius**2 * 2

class Cube(RectangularPrism):
    'Class representing a cube.'
    
    def __init__(self,color,side):
        super().__init__(color,side,side,side)
    
    def __repr__(self):
        return f"{Shape.__repr__(self).replace('shape','cube')}, with sides of length {self.length}"
    
    # question #2
    def __add__(self,other):
        return Cube(self.color,self.length + other.length)